'use strict'

const Stream = require('stream')
const queryString = require('querystring')

module.exports = (event, callback) => {
  const response = {
    body: '',
    statusCode: 200,
    headers: {}
  }

  const req = new Stream.Readable()
  req._read = f => f
  req.url = event.requestContext.path.replace(
    new RegExp('^/' + event.requestContext.stage),
    ''
  )
  if (event.queryStringParameters) {
    req.url += '?' + queryString.stringify(event.queryStringParameters)
  }
  req.method = event.httpMethod
  req.rawHeaders = {}
  req.headers = {}

  for (const key of Object.keys(event.headers)) {
    req.rawHeaders[key] = event.headers[key]
    req.headers[key.toLowerCase()] = event.headers[key]
  }

  req.connection = {}

  const res = new Stream()
  res.headers = {}
  res.writeHead = (status, headers) => {
    response.statusCode = status
    if (headers) res.headers = Object.assign(res.headers, headers)
  }
  res.write = chunk => {
    response.body += chunk
  }
  res.setHeader = (name, value) => {
    res.headers[name] = value
  }
  res.end = text => {
    if (text) response.body += text
    response.headers = res.headers || {}
    callback(null, response)
  }
  if (event.body) {
    req.push(event.body)
    req.push(null)
  }

  return { req, res }
}
